import base64
import cv2
import json
import logging
import math
import numpy as np
import os
import time
from datetime import datetime
from fischertechnik.mqtt.Constants import CONTROLLER_ID
from fischertechnik.mqtt.FTCloudClient import FTCloudClient
from fischertechnik.mqtt.MqttClient import MqttClient
from lib.camera import *
from lib.controller import *
from lib.display import *
from lib.display_update import *
from lib.Factory_Variables import *
from lib.Time import *

message = None
camera_enabled = None
camera_image = None
payload_temp = None
payload_bme680 = None
payload_ldr = None
last_image_sent = None
payload_broadcast = None
last_humidity_alarm = None
last_temperature_alarm = None
last_movement_alarm = None
controller_name = None
payload_cam = None


def publish_bme680():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    while True:
        payload_bme680 = '{{"ts":"{}","t":{:.1f},"rt":{:.1f},"h":{:.1f},"rh":{:.1f},"p":{:.1f},"iaq":{},"aq":{},"gr":{}}}'.format(timestamp_utcnow(), (TXT_M_I2C_1_environment_sensor.get_temperature()) - 4, 0, TXT_M_I2C_1_environment_sensor.get_humidity(), 0, TXT_M_I2C_1_environment_sensor.get_pressure(), TXT_M_I2C_1_environment_sensor.get_indoor_air_quality_as_number(), TXT_M_I2C_1_environment_sensor.get_accuracy(), 0)
        if get_cloud_active():
            FTCloudClient.getInstance().publish("/j1/txt/" + str(CONTROLLER_ID) +"/i/bme680", payload_bme680)
        if (get_client_local()) != None:
            get_client_local().publish(topic='i/bme680', payload=payload_bme680, qos=0, retain=False)
        time.sleep((get_bme680_period()))


def publish_ldr():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    while True:
        payload_ldr = '{{"ts":"{}", "br":{:.1f}, "ldr":{}}}'.format(timestamp_utcnow(), round((65000 - TXT_M_I3_photo_resistor.get_resistance()) / 650, 1), TXT_M_I3_photo_resistor.get_resistance())
        if get_cloud_active():
            FTCloudClient.getInstance().publish("/j1/txt/" + str(CONTROLLER_ID) +"/i/ldr", payload_ldr)
        if (get_client_local()) != None:
            get_client_local().publish(topic='i/ldr', payload=payload_ldr, qos=0, retain=False)
        time.sleep((get_ldr_period()))


def publish_camera():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    camera_image = []
    last_image_sent = False
    while True:
        if not not len(camera_image):
            payload_cam = '{{"ts":"{}","data":"{}"}}'.format(timestamp_utcnow(), frame_to_base64(camera_image))
            if (get_client_local()) != None:
                get_client_local().publish(topic='i/cam', payload=payload_cam, qos=0, retain=False)
            if get_cloud_active():
                if gibt_switch_camera_status():
                    if last_image_sent:
                        last_image_sent = False
                    else:
                        FTCloudClient.getInstance().publish("/j1/txt/" + str(CONTROLLER_ID) +"/i/cam", payload_cam)
                        last_image_sent = True
        time.sleep((1 / (get_camera_fps())))


def camera_enable():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    camera_enabled = True


def camera_enabled2():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    return camera_enabled


def camera_disable():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    camera_enabled = False


def callback(event):
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    if TXT_M_I2C_1_environment_sensor.get_humidity() > 80:
        logging.log(logging.TRACE0_FCL, '-')
        if time.time() - last_humidity_alarm >= (get_alarm_timer()):
            publish_humidity_alarm()
            last_humidity_alarm = time.time()



def image_callback(event):
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    if camera_enabled:
        camera_image = event.value
    else:
        camera_image = []


def motion_callback(event):
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    if camera_enabled:
        logging.log(logging.TRACE0_FCL, '-')
        if last_movement_alarm != None:
            if time.time() - last_movement_alarm >= (get_alarm_timer()):
                publish_movement_alarm()
                last_movement_alarm = time.time()


def callback2(event):
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    if TXT_M_I2C_1_environment_sensor.get_temperature() < 4:
        logging.log(logging.TRACE0_FCL, '-')
        if time.time() - last_temperature_alarm >= (get_alarm_timer()):
            publish_temperature_alarm()
            last_temperature_alarm = time.time()



def publish_movement_alarm():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    FTCloudClient.getInstance().publish("/j1/txt/" + str(CONTROLLER_ID) +"/i/alert", '{{"ts":"{}","id":"{}","data":"{}","code":{}}}'.format(timestamp_utcnow(), 'cam', frame_to_base64(camera_image), '100'))
    print('Clould Image alarm')


def publish_temperature_alarm():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    FTCloudClient.getInstance().publish("/j1/txt/" + str(CONTROLLER_ID) +"/i/alert", '{{"ts":"{}","id":"{}","data":"{}","code":{}}}'.format(timestamp_utcnow(), 'bme680/t', TXT_M_I2C_1_environment_sensor.get_temperature(), '200'))


def publish_humidity_alarm():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    FTCloudClient.getInstance().publish("/j1/txt/" + str(CONTROLLER_ID) +"/i/alert", '{{"ts":"{}","id":"{}","data":"{}","code":{}}}'.format(timestamp_utcnow(), 'bme680/h', TXT_M_I2C_1_environment_sensor.get_humidity(), '300'))


def publish_broadcast(message):
    global camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    controller_name = os.uname()[1]
    payload_broadcast = '{{"ts":"{}","hardwareId":"{}","hardwareModel":"{}","softwareName":"{}","softwareVersion":"{}","message":"{}"}}'.format(timestamp_utcnow(), controller_name, 'TXT 4.0', 'GatewayPLC', display.get_attr("txt_label_version.text"), message)
    if get_cloud_active():
        FTCloudClient.getInstance().publish("/j1/txt/" + str(CONTROLLER_ID) +"/i/broadcast", payload_broadcast)
    if (get_client_local()) != None:
        get_client_local().publish(topic='i/broadcast', payload=payload_broadcast, qos=2, retain=False)


def publish_config_local():
    global message, camera_enabled, camera_image, payload_temp, payload_bme680, payload_ldr, last_image_sent, payload_broadcast, last_humidity_alarm, last_temperature_alarm, last_movement_alarm, controller_name, payload_cam
    logging.log(logging.TRACE_FCL, '-')
    payload_temp = '{{"ts":"{}","period":3}}'.format(timestamp_utcnow())
    if (get_client_local()) != None:
        get_client_local().publish(topic='c/bme680', payload=payload_temp, qos=2, retain=False)
        get_client_local().publish(topic='c/ldr', payload=payload_temp, qos=2, retain=False)
    payload_temp = '{{"ts":"{}","on":true,"fps":2}}'.format(timestamp_utcnow())
    if (get_client_local()) != None:
        get_client_local().publish(topic='c/cam', payload=payload_temp, qos=2, retain=False)


TXT_M_I2C_1_environment_sensor.add_change_listener("humidity", callback)
TXT_M_USB1_1_camera.add_change_listener("image", image_callback)
TXT_M_I2C_1_environment_sensor.add_change_listener("temperature", callback2)

motion_detector.add_detection_listener(motion_callback)

def frame_to_base64(frame):
        result = ""
        success, image = cv2.imencode(".jpeg", frame, [1, 30])
        if success:
                result = "data:image/jpeg;base64," + base64.b64encode(image).decode("utf-8")
        return result

